' MMBasic v4.0 Sample
'
' LIBRARY CODE for driving JY-MCU / JY-LKM1638 module availble from:
'  http://dx.com/p/8x-digital-tube-8x-key-8x-double-color-led-module-81873
'
' Code copyright 2012 John Harding
' Beer License: if this is of any value to you and our paths should cross
' then buy me a beer.
'
' Modified to access the 7 Segment display modules as well
' April 2013 by Mick Gulovsen
'
' I owe John a Beer...
'


' you can choose any three pins you want...
CLK = 13 : DIO = 14 : STB = 15

' Setup pins and initialize the module
Sub TMSetup
  SetPin CLK,8 : SetPin DIO,8 : SetPin STB,8
  Pin(STB)=1 : Pin(CLK)=1
  TMSendCmd(&h40) : TMSendCmd(&h8F)
  Pin(STB)=0
  TMSend(&hC0)
  Local i ' strange that I have to declare local i for this for/next to work
  For i=0 To 15
    TMSend(&h00) 'zero out everything
  Next i
  Pin(STB)=1
End Sub

' Send one byte to the module
' note that this routine produces a clock signal with a "lop-sided" duty cycle
' however the module doesn't seem to care.
Sub TMSend(data)
  j = 1
  For i=0 To 7
    Pin(CLK)=0
    If (data And j) Then
      Pin(DIO)=1
    Else
      Pin(DIO)=0
    EndIf
    j = j * 2 ' this is an "expensive operation" is there a better way?
    Pin(CLK)=1
  Next i
End Sub

' Send a command byte to the module
Sub TMSendCmd(cmd)
  Pin(STB)=0
  TMSend(cmd)
  Pin(STB)=1
End Sub

' Send a data byte to the module
Sub TMSendData(address, data)
  TMSendCmd(&h44)
Pin(STB)=0
  TMSend(&hC0 Or address)
  TMSend(data)
  Pin(STB)=1
End Sub

' Set the LED number (0 based) to the given color
'  c=1 - red
'  c=2 - green
'  c=3 - red + green = orange
'  c=any other value switches off LED
Sub TMSetLED(p, c)
  TMSendData( p*2+1, c )
End Sub

' Set the 7Seg number (0 based) to the given number
' 0  &b00111111
' 1  &b00000110
' 2  &b01011011
' 3  &b01001111
' 4  &b01100110
' 5  &b01101101
' 6  &b01111101
' 7  &b00000111
' 8  &b01111111
' 9  &b01101111
Sub TMSet7(p, c)
  TMSendData( p*2, c )
End Sub

Sub Strip(p$)
For y=1 To Len(p$)
  y$=Mid$(p$,y,1)
If y$="0" Then clr= &b00111111
If y$="1" Then clr= &b00000110
If y$="2" Then clr= &b01011011
If y$="3" Then clr= &b01001111
If y$="4" Then clr= &b01100110
If y$="5" Then clr= &b01101101
If y$="6" Then clr= &b01111101
If y$="7" Then clr= &b00000111
If y$="8" Then clr= &b01111111
If y$="9" Then clr= &b01101111
TMSet7(y-1,clr)
Next y

End Sub

''''''''''''''''''''''''''''''''''''''''''''''''
' DEMO code for the above library...

Demo:
TMSetup()
Cls
? "Demo Code for driving JY-MCU / JY-LKM1638 module (8x7Seg/TriLED/Switches)"
? "Availble from:   http://dx.com    sku  81873"

?:?:? "        Select Test Required (esc) to exit"
?:?:? "        Press L for LED Test"
?"        Press 7 for 7Seg Test "
Do
k$=Inkey$
If k$="l" Then GoTo Ledz
If k$="L" Then GoTo Ledz
If k$="7" Then GoTo Seg7
If k$="s" Then GoTo Switches
If k$="S" Then GoTo Switches
If k$=Chr$(27) Then End
Loop

Ledz:
Cls
? "watch the pretty lights and press  (esc) when bored"
? "Colour 0=Off, Colour 1=RED, Colour 2=Green, Colour 3=Orange (near RED)":?

led = 0
clr = 0
Do
  If Inkey$=Chr$(27) Then GoTo Demo
  TMSetLED(led, clr)
   ? "Colour " clr;" For LED ";led;" "
   Pause(200)
  led = led + 1 : If led > 7 Then led = 0:?
  If led = 0 Then
    clr = clr + 1 : If clr > 3 Then clr = 0:GoTo Ledz
  EndIf
Loop

Seg7:


Cls

? "7 Segment display Demo Press (esc) to return to menu":?:?
? "Press c to Select count 0-99999999"
? "Press z to Select count 0-99999999 with leading zeros"

Do
k$=Inkey$
If k$="c" Then zero=0: GoTo Seg7cnt
If k$="C" Then zero=0: GoTo Seg7cnt
If k$="z" Then zero=1: GoTo Seg7cnt
If k$="Z" Then zero=1: GoTo Seg7cnt
Loop

Seg7cnt:
seg = 0
' clr = &b00111111

For x=0 To 99999999
  x$=Str$(x)
If zero=0 Then GoTo Seg7skip

AddZero:

  If Len(x$) <8 Then x$ = "0"+x$ : GoTo AddZero Else Strip(x$)

Seg7skip:

If Inkey$=Chr$(27) Then GoTo Demo

Strip(x$)

Next x

End

Do
  TMSet7(seg, clr)
  seg = seg + 1 : If seg > 7 Then seg = 0
Loop

